#!/usr/bin/perl
use strict;
use warnings;
use Math::MPFI qw(:mpfi);

Rmpfi_set_default_prec(64);

sub find_fixed_point{
  my ($fref, $x0) = @_;
  if (Rmpfi_is_empty($x0)) {
    return "$x0 is empty\n";
  } 
  my $x1 = $fref->($x0);
  if (!Rmpfi_is_inside($x1, $x0)) {
    return "$x1 not in $x0\nf(x0) not include in x0\n";
  }  
  Rmpfi_intersect($x1, $x1, $x0);   
  do {
    $x0 = $x1;
    $x1 = $fref->($x0);
    Rmpfi_intersect($x1, $x1, $x0);   
  } until (Rmpfi_is_inside($x0, $x1));
  return "x* = $x1\n";
}

sub f{ return 2+$_[0]/2; }
my $x0 = Math::MPFI->new('[3,2]');
print "f(x)=2+x/2 and x0=$x0\n";
print find_fixed_point(\&f, $x0), "\n";

$x0 = Math::MPFI->new('[1,2]');
print "f(x)=2+x/2 and x0=$x0\n";
print find_fixed_point(\&f, $x0), "\n";

$x0 = Math::MPFI->new('[2,5]');
print "f(x)=2+x/2 and x0=$x0\n";
print find_fixed_point(\&f, $x0), "\n";

sub g{ return exp(-$_[0]); }
$x0 = Math::MPFI->new('[-@Inf@,@Inf@]');
print "g(x)=exp(-x) and x0=$x0\n";
print find_fixed_point(\&g, $x0), "\n";
